use std::sync::Arc;

use anyhow::Context;

#[derive(Clone)]
pub struct Nats {
    nats: Arc<nats::asynk::Connection>,
}

impl Nats {
    pub async fn new() -> anyhow::Result<Self> {
        let nats = nats::asynk::Options::with_user_pass(
            &std::env::var("NATS_USERNAME").context("NATS_USERNAME was not found")?,
            &std::env::var("NATS_PASSWORD").context("NATS_PASSWORD was not found")?,
        )
        .with_name(std::env!("CARGO_PKG_NAME"))
        .connect(std::env::var("NATS_URL").context("NATS_URL was not found")?)
        .await?;

        Ok(Self {
            nats: Arc::new(nats),
        })
    }
}

impl std::ops::Deref for Nats {
    type Target = nats::asynk::Connection;

    fn deref(&self) -> &Self::Target {
        &self.nats
    }
}
